// 683-LedPWM.c
//
// Universal (almost) low-voltage dimmer / fan control
//
// Target:        LEDPWM3 (Custom PCB) PIC12F683
// Compiler:      CCS PCWH 4.082
// Programmer:    Microchip PicKit2           
//
// Copyright (c) 2010 Jurgen G Schmidt
//
// 2007.11.14 - Original development with PicBasicPro
// 2010.02.03 - Revisit - make new boards and translate to CCS PCWH 4.082
// 2010.02.04 - 16khz PWM frequency eliminates fan motor whining
//              fan needs at least 60% to start
//            - save last setting in eeprom and reload it at start
// 2010.02.05 - differences for fan and LED light - everything working
// 2010.03.06 - Button held on power-up programs fan or LED mode
//              (maxPWM and minPWM seem to be reserved words)
//
// Pin 1 Vdd (Positive) 
// Pin 2 GP5 - Dim
// Pin 3 GP4 - 
// Pin 4 GP3/MCLR - save for ICSP
// Pin 5 GP2 - PWM output -controlled by Timer2
// pin 6 GP1 - ICSPCLK & serial output for debugging
// pin 7 GP0 - ICSPDAT Brighten
// Pin 8 Vss (ground)

//-----------------------------------------------------------------------------
// a few internal eeprom macro wrappers from M Bradley
// http://www.ccsinfo.com/forum/viewtopic.php?t=40941
#define mReadByte(addr)         read_eeprom(addr) 
#define mWriteByte(addr,data)   write_eeprom(addr,data) 
// little endian 
#define mReadWord(addr)         make16( read_eeprom(addr + 1), read_eeprom(addr) ); 
#define mWriteWord(addr, data)  { write_eeprom(addr, make8(data,0)); write_eeprom(addr + 1, make8(data,1)); }
//-----------------------------------------------------------------------------

#define  DEBUG     // uncomment for serial output & heartbeat

#include <12f683.h>
#fuses INTRC_IO,NOMCLR,NOWDT,NOPROTECT
#use delay( internal=8M )
#use rs232 (invert, baud=9600, xmit=PIN_A1)

#rom  0x2100={250,0,0}        // initialize EEPROM to 50%, fan 
                              // only happens with program update

#define  pin_HI   PIN_A0      // increast duty
#define  pin_LO   PIN_A5      // decrease duty
#define  pin_LED  PIN_A1      // heartbeat LED

unsigned int16 duty;          // range= 0-500 
unsigned int16 max_pwm;       // what is max duty value allowed
unsigned int16 min_pwm;       // what is min duty allowed
unsigned int   step;          // how big a change for each button push
unsigned int   mode;          // 0=FAN 1=LED

//-----------------------------------------------------------------------------
void brighter()
{
#ifdef DEBUG
   output_low(PIN_LED);    // show we actually pushed a button
   delay_ms(100);
   output_high(PIN_LED);
   delay_ms(100);
#endif

   duty += step;
   if (duty >= max_pwm) 
   {
      duty = max_pwm;
      mWriteWord(0,duty);
      return;
   }
   mWriteWord(0,duty);
   return;
}
//-----------------------------------------------------------------------------
void dimmer()
{
#ifdef DEBUG
   output_low(PIN_LED);    // show we actually pushed a button
   delay_ms(100);
   output_high(PIN_LED);
   delay_ms(100);
#endif

   if (duty == min_pwm) return;
   duty -= step;
   mWriteWord(0,duty);
   return;
}
//-----------------------------------------------------------------------------
void main()
{
   output_low(PIN_A2);     // turn off transistor
   delay_ms(100);          // let everything settle

#ifdef DEBUG
   output_low(PIN_LED);    // we're awake...
   delay_ms(500);
   output_high(PIN_LED);
   delay_ms(500);
#endif

   // check for held button...
   if (input(PIN_LO) == 0)  mWriteByte( 2, 0 ); // set for FAN
   if (input(PIN_HI) == 0)  mWriteByte( 2, 1 ); // set for LED
   
   //--- set up PWM and timer
   //--- see spreadsheet. At 8Mhz is 16khz with resolution of 500 steps...
   setup_ccp1(CCP_PWM);    
   setup_timer_2(T2_DIV_BY_1, 124, 1);
   set_pwm1_duty(0);       // initialize off

   duty = mReadWord( 0 );  // reload duty value from EEPROM on startup
   mode = mReadByte( 2 );  // what mode
   
   if( mode==0 ) {      // fan settings
#ifdef DEBUG
      printf("hello LedPWM for Fan...\n");
#endif      
      min_pwm = 100;     // fan won't go below a certain point 
      max_pwm = 500;
      step = 50;
      
      // kick-start fan if set at low speed
      if (duty < 400) 
      {
         set_pwm1_duty(400);
         delay_ms(1000);
      }
   } else {             // Light settings
#ifdef DEBUG
      printf("hello LedPWM for LED...\n");
#endif      
      min_pwm = 0;
      max_pwm = 500;
      step = 10;        // Light has finer resolution so 50 steps
   }
         
#ifdef DEBUG
   printf("Read: %ld  %d\n",duty, mode);
#endif   

   while( TRUE )
   {
#ifdef DEBUG   
      printf("Duty: %ld \n",duty);
#endif

      set_pwm1_duty(duty);
      
      if (input(PIN_HI) == 0) brighter();
      if (input(PIN_LO) == 0) dimmer();

      delay_ms(200);     // slow things down
   }
}
//-----------------------------------------------------------------------------
//------ end of 683-LedPWM.c ------

